---------------------------
-- adder_1
---------------------------

library ieee;
use ieee.std_logic_1164.all;

entity adder_1 is
  generic(
    td_g: time :=5 ns
  ); port(
    d2_p:     std_logic;
    d1_p:     std_logic;
    d0_p:     std_logic;
    s1_p: out std_logic;
    s0_p: out std_logic
  ); 
end entity adder_1;

architecture beh of adder_1 is
  signal input:  std_logic_vector(0 to 2);
  signal output: std_logic_vector(0 to 1);
begin
  input <= (d0_p,d1_p,d2_p);

  with input select  
    output <= 
      "00" after td_g when "000",
      "01" after td_g when "001",
      "01" after td_g when "010",
      "10" after td_g when "011",
      "01" after td_g when "100",
      "10" after td_g when "101",
      "10" after td_g when "110",
      "11" after td_g when "111",
      "UU" after td_g when others;

  (s1_p,s0_p) <= output;

end architecture beh;


---------------------------
-- adder_1_t
---------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_textio.all;
use std.textio.all;

entity adder_1_t is
end entity adder_1_t;

architecture test of adder_1_t is
  signal d0:   std_logic;
  signal d1:   std_logic;
  signal cin:  std_logic;
  signal s:    std_logic;
  signal cout: std_logic;

  procedure printStatus is
    variable l:line;
  begin
    write(l, now, right, 10, ns);
    write(l, string'(" in: d0,d1,cin="));
    write(l, std_logic_vector'(d0,d1,cin));
    write(l, string'(", out: cout,s="));
    write(l, std_logic_vector'(cout,s));
    writeline(output, l);
  end procedure printStatus;

begin
  t: entity work.adder_1 
    generic map(
      td_g => 5 ns)
    port map (
      d0_p   => d0,
      d1_p   => d1,
      d2_p   => cin,
      s0_p   => s,
      s1_p   => cout);

  test: process 
    constant base:integer :=
      std_logic'pos('0');
  begin
    for i in 0 to 1 loop
      for j in 0 to 1 loop
        for k in 0 to 1 loop
          d0  <= std_logic'val(base+i);
          d1  <= std_logic'val(base+j);
          cin <= std_logic'val(base+k);
          wait for 10 ns; 
          printStatus;
        end loop;
      end loop;
    end loop;

    wait;
  end process;
end architecture test;

