---------------------------
-- dfa
---------------------------

library ieee;
use ieee.std_logic_1164.all;

entity dfa is
  generic(
    td_g: time :=100 ns
  ); port(
    reset_p: in  std_logic;
    a_p:     in  std_logic;
    clk_p:   in  std_logic;
    z_p:     out std_logic
  );
end entity dfa;

architecture beh of dfa is
  subtype state is 
    std_logic_vector(1 downto 0);
  signal s_s: state;
  
  procedure switchState(
    a: std_logic; s: state; 
    snew: out state; znew: out std_logic)
  is
    variable input: 
      std_logic_vector(2 downto 0):=
      (s(1), s(0), a);
    variable output: 
      std_logic_vector(2 downto 0);
  begin
    case input is 
      when "000"  => output := "000"; 
      when "001"  => output := "110"; 
      when "010"  => output := "010"; 
      when "011"  => output := "110"; 
      when "100"  => output := "101"; 
      when "101"  => output := "011"; 
      when "110"  => output := "101"; 
      when "111"  => output := "001"; 
      when others => output := "UUU"; 
    end case;
    snew:=output(2 downto 1);
    znew:=output(0);
  end switchState;
    
begin
  ff: process(clk_p, reset_p) 
    variable snext: state;
    variable znext: std_logic;
  begin
    if (rising_edge(reset_p)) then
      z_p <= '0';
      s_s <= "00";
    elsif (rising_edge(clk_p)) then
      switchState(a_p, s_s, snext, znext);
      z_p <= znext;
      s_s <= snext;
    end if;
  end process;
end architecture beh;


---------------------------
-- dfa_t
---------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_textio.all;
use std.textio.all;

entity dfa_t is
end entity dfa_t;

architecture test of dfa_t is
  signal a:    std_logic;
  signal z:    std_logic;
  signal clk:  std_logic;
  signal reset:std_logic;

  procedure printStatus is
    variable l:line;
  begin
	  write(l, now, right, 10, ns);
    write(l, string'(" in="));
    write(l, a);
    write(l, string'(", clk="));
    write(l, clk);
    write(l, string'(" out="));
    write(l, z);
    writeline(output, l);
  end procedure printStatus;

begin
  test_dfa: entity work.dfa(beh)
    generic map(
      td_g => 5 ns)
    port map (
      reset_p => reset,
      a_p     => a,
      clk_p   => clk,
      z_p     => z);

  test: process 
    constant td:time := 200 ns;
  begin
  
    reset <= '0';wait for td;
    reset <= '1';wait for td;
    reset <= '0';wait for td;

    a <= '1'; 
    clk <= '0'; wait for td; 
    clk <= '1'; wait for td;
    printStatus;
    assert(z='0') report 
      "test failed on 00->11";

    a <= '0'; 
    clk <= '0'; wait for td; 
    clk <= '1'; wait for td;
    printStatus;
    assert(z='1') report 
      "test failed on 11->10";

    a <= '1'; 
    clk <= '0'; wait for td; 
    clk <= '1'; wait for td;
    printStatus;
    assert(z='1') report 
      "test failed on 10->01";

    clk <= '0'; wait for td; 
    clk <= '1'; wait for td;
    printStatus;
    assert(z='0') report 
      "test failed on 01->11";

    wait;
  end process test;

end architecture test;

