-------------------------------
-- parity_n
-------------------------------

library ieee;
use ieee.std_logic_1164.all;
 
entity parity_n is
    generic(
      n_g: integer :=10
    ); port(
      d_p  : in  std_logic_vector(0 to n_g-1);
      odd_p: out std_logic
    );
end parity_n;
 
architecture sequential of parity_n is
  signal p: std_logic_vector(0 to n_g - 3);
begin
  assert n_g>=3;
  
  p(0) <= d_p(0) xor d_p(1);

  G: for I in 1 to (n_g - 3) generate
    p(i) <= p(i-1) xor d_p(i+1);
  end generate G;
  
  odd_p <= p(n_g-3) xor d_p(n_g-1);
 
end sequential;


-- architecture tree of parity_n is
--  signal p1: std_logic;  
--  signal p2: std_logic;  
--  component parity_n is
--      generic(
--        n_g: integer :=10
--      ); port(
--        d_p  : in  std_logic_vector(0 to n_g-1);
--        odd_p: out std_logic
--      );
--  end component;
--begin
--  assert n_g>=2;
--
--  g2: if n_g=2 generate
--    odd_p <= d_p(0) xor d_p(1);
--  end generate;
--
--  g3: if n_g=3 generate
--    odd_p <= d_p(0) xor d_p(1) xor d_p(2);
--  end generate;
--
--  gn: if n_g>3 generate
--    g1: entity work.parity_n
--      generic map(n_g/2)
--      port map (d_p(0 to n_g/2-1),p1);
--    g2: entity work.parity_n
--      generic map(n_g-n_g/2)
--      port map (d_p(n_g/2-1 to n_g-1),p2);
--      odd_p <= p1 xor p2;
--  end generate;
--end tree;


-------------------------------
-- parity_n_t
-------------------------------

library ieee;
use ieee.std_logic_1164.all;
use std.textio.all;
use ieee.std_logic_textio.all;
 
entity parity_n_t is
end parity_n_t;
 
 
architecture test of parity_n_t is
  constant n: integer :=4;
  signal d: std_logic_vector(0 to n-1);
  signal odd: std_logic;

  type test_t is record
      d   : std_logic_vector (0 to n-1);
      odd : std_logic;
  end record;
  type test_vector_t is array(positive range <>) of test_t;
  constant testvec : test_vector_t := (
    ("0001",'1'),
    ("0101",'0'),
    ("0111",'1'),
    ("0011",'0')
  );
begin
 
  uut: entity work.parity_n
    generic map(4)
    port map (D,odd);

  test: process 
    variable test : test_t;
    variable l:line;
  begin
    for index in testvec'range loop
      test := testvec(index);

      -- apply the input stimulus...
      d <= test.d; 
      wait for 50 ns;

      -- print results
      write(l, string'(" index: "));
      write(l, index);
      write(l, string'(" stimulus: "));
      write(l, test.d);
      write(l, string'(" odd: "));
      write(l, odd);
      writeline(output, l);

      -- check the results...
      if (test.odd /= odd) then
         assert false 
           report "Output did not match!" 
           severity WARNING;
      end if;
    end loop;
    wait;
  end process;
end test;

